/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

struct VS_MESH
{
	float4 CurrPosition: POSITION0;
	float3 CurrNormal : NORMAL0;
	float4 PrevPosition: POSITION1;
	float3 PrevNormal : NORMAL1;
	float2 TexCoord: TEXCOORD0;
};


struct PS_MESH
{
	float4 Position: POSITION0;
	float2 TexCoord: TEXCOORD0;
	float3 Normal : TEXCOORD1;
};


#ifdef VERTEXSHADER
#include "fxVSIncludes.fx"

float4 GetMeshPosition (VS_MESH vs_in)
{
	return float4 (MeshMove + vs_in.CurrPosition.xyz * MeshFrontV + vs_in.PrevPosition.xyz * MeshBackV, 1.0f);
}

float3 GetMeshNormal (VS_MESH vs_in)
{
	return lerp (vs_in.CurrNormal, vs_in.PrevNormal, MeshBlend);
}

PS_MESH MeshTexturedVS (VS_MESH vs_in)
{
	PS_MESH vs_out;

	vs_out.Position = GetPosition (EntityMatrix, GetMeshPosition (vs_in));
	vs_out.TexCoord = vs_in.TexCoord;
	vs_out.Normal = GetMeshNormal (vs_in);

	return vs_out;
}

PS_MESH MeshShellVS (VS_MESH vs_in)
{
	PS_MESH vs_out;

	float3 lerpnormal = GetMeshNormal (vs_in);

	vs_out.Position = GetPosition (EntityMatrix, GetMeshPosition (vs_in) + float4 (lerpnormal * MeshSuitScale, 0.0f));
	vs_out.TexCoord = vs_in.TexCoord;	// unused
	vs_out.Normal = lerpnormal;			// unused

	return vs_out;
}

PS_GENERICDYNAMIC MeshDynamicVS (VS_MESH vs_in)
{
	// these don't get powersuit effects
	return GenericDynamicVS (
		GetMeshPosition (vs_in),
		GetMeshNormal (vs_in),
		vs_in.TexCoord
	);
}
#endif


#ifdef PIXELSHADER
#include "fxPSIncludes.fx"

#define meshSampler tmu0Sampler

float4 MeshTexturedPS (PS_MESH ps_in) : COLOR0
{
	// per-vertex normals need to be renormalized to bring them back to unit length
	float shadedot = dot (normalize (ps_in.Normal), shadevector);

	float4 diff = ApplyGammaPS (tex2D (meshSampler, ps_in.TexCoord), Intensity);
	float4 lmap = float4 (shadelight.rgb * max (shadedot + 1.0f, (shadedot * 0.2954545f) + 1.0f), 1.0f);

	return float4 (diff.rgb * lmap.rgb, Alpha.a);
}

float4 MeshShellPS (PS_MESH ps_in) : COLOR0
{
	return ApplyGammaPS (float4 (shadelight.rgb, Alpha.a), Intensity);
}

float4 MeshDynamicPS (PS_GENERICDYNAMIC ps_in) : COLOR0
{
	return GenericDynamicPS (ps_in);
}
#endif

