/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

struct VS_PARTICLE
{
	float4 Position: POSITION0;
	float4 Color : COLOR0;
	float Alpha : TEXCOORD1;
	float2 Offsets: TEXCOORD0;
};


struct PS_PARTICLE
{
	float4 Position: POSITION0;
	float4 Color : COLOR0;
	float2 Offsets: TEXCOORD0;
};


#ifdef VERTEXSHADER
#include "fxVSIncludes.fx"

PS_PARTICLE PartVS (VS_PARTICLE vs_in)
{
	PS_PARTICLE vs_out;

	float2 scale = vs_in.Offsets * (1.0f + dot (vs_in.Position.xyz - r_origin, vpn) * 0.002f);
	float4 OutPos = float4 (vs_in.Position.xyz + (vright * scale.x + vup * scale.y) * 0.666f, 1.0f);

	vs_out.Position = GetPosition (WorldMatrix, OutPos);

	vs_out.Color = float4 (vs_in.Color.rgb, vs_in.Alpha);
	vs_out.Offsets = vs_in.Offsets;

	return vs_out;
}
#endif


#ifdef PIXELSHADER
#include "fxPSIncludes.fx"

float4 PartPS (PS_PARTICLE ps_in) : COLOR0
{
	float alpha = min ((1.0f - dot (ps_in.Offsets, ps_in.Offsets)) * 1.5f, 1.0f);
	return ApplyGammaPS (float4 (ps_in.Color.rgb, ps_in.Color.a * alpha), 1.0f);
}
#endif

