/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// separate because it needs to be shared by C and C++ code
#ifdef __cplusplus
extern "C" {
#ifndef SAFE_RELEASE
#define SAFE_RELEASE(obj) if (obj) {obj->Release (); obj = NULL;}
#endif
#endif

#ifndef __d3dcommon_h__
// make this struct available to the C code too
typedef struct _D3D_SHADER_MACRO
{
	LPCSTR Name;
	LPCSTR Definition;
} D3D_SHADER_MACRO;
#endif

// vertex shader constant register slots
#define VSC_WORLDMATRIX			0

#define VSC_ENTITYMATRIX		4
#define VSC_LOCALMATRIX			8

#define VSC_WARPTIME			12
#define VSC_VUP					13
#define VSC_VPN					14
#define VSC_VRIGHT				15
#define VSC_RORIGIN				16

#define VSC_SPRITEORIGIN		17
#define VSC_TEXSCROLL			18
#define VSC_SKYMATRIX			19
#define VSC_TRANSFORMEDDL		23
#define VSC_SURFACEPASS			24

#define VSC_ORTHOMATRIX			25
#define VSC_RTINVERSE			29

#define VSC_MESHMOVE			30
#define VSC_MESHFRONTV			31
#define VSC_MESHBACKV			32
#define VSC_MESHSUITSCALE		33
#define VSC_MESHBLEND			34

#define VSC_LIGHTSTYLES			128

// pixel shader constant register slots
#define PSC_POLYBLEND			0
#define PSC_SHADELIGHT			1
#define PSC_SHADEVECTOR			2
#define PSC_LIGHTCOLOR			3
#define PSC_LIGHTRADIUS			4
#define PSC_ALPHAVAL			5
#define PSC_WARPTIME			6
#define PSC_GAMMA				7
#define PSC_INTENSITY			8
#define PSC_BRIGHTNESS			9

// texture register slots
#define TS_DIFFUSE		0
#define TS_LIGHT		1
#define TS_SKY			2
#define TS_SCREEN		3
#define TS_POSTPROC		4

// exports
IDirect3DVertexShader9 *R_CreateVertexShader (int resourceID, const char *entrypoint);
IDirect3DPixelShader9 *R_CreatePixelShader (int resourceID, const char *entrypoint);
IDirect3DVertexDeclaration9 *R_CreateVertexDeclaration (D3DVERTEXELEMENT9 *layout);

void R_ReleaseShaders (void);
void R_SetObjectName (IDirect3DResource9 *pObject, char *name);

void R_SetVertexShaderConstant4fv (int reg, float *c, int num4f);
void R_SetVertexShaderConstant4f (int reg, float c0, float c1, float c2, float c3);
void R_SetVertexShaderConstant3f (int reg, float c0, float c1, float c2);
void R_SetVertexShaderConstant2f (int reg, float c0, float c1);
void R_SetVertexShaderConstant1f (int reg, float c0);
void R_SetPixelShaderConstant4fv (int reg, float *c, int num4f);
void R_SetPixelShaderConstant4f (int reg, float c0, float c1, float c2, float c3);
void R_SetPixelShaderConstant3f (int reg, float c0, float c1, float c2);
void R_SetPixelShaderConstant2f (int reg, float c0, float c1);
void R_SetPixelShaderConstant1f (int reg, float c0);

// creates a shader pair (vs and ps) using the standard naming convention; assumes no consts and both are in the same source resource
#define R_CreateShaders(vs, ps, res) \
	d3d_##vs = R_CreateVertexShader (res, #vs); \
	d3d_##ps = R_CreatePixelShader (res, #ps);

#ifdef __cplusplus
}
#endif

