/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#pragma once

#include "versions.h"
#include <windows.h>

#include <stdio.h>
#include <stddef.h>
#include <stdint.h>

#include <d3d9.h>
#include <math.h>

#include "ref.h"
#include "qgl.h"

#include "gl_cpplayer.h"

#define	REF_VERSION	"GL 0.01"

// up / down
#define	PITCH	0

// left / right
#define	YAW		1

// fall over
#define	ROLL	2

extern	viddef_t	vid;

// up here so that it's available to the model code
#ifndef __D3DX9TEX_H__
// define D3DX constants missing since we no longer have D3DX available
#define D3DX_FILTER_NONE             (1 << 0)
#define D3DX_FILTER_POINT            (2 << 0)
#define D3DX_FILTER_LINEAR           (3 << 0)
#define D3DX_FILTER_TRIANGLE         (4 << 0)
#define D3DX_FILTER_BOX              (5 << 0)

#define D3DX_FILTER_MIRROR_U         (1 << 16)
#define D3DX_FILTER_MIRROR_V         (2 << 16)
#define D3DX_FILTER_MIRROR_W         (4 << 16)
#define D3DX_FILTER_MIRROR           (7 << 16)

#define D3DX_FILTER_DITHER           (1 << 19)
#define D3DX_FILTER_DITHER_DIFFUSION (2 << 19)

#define D3DX_FILTER_SRGB_IN          (1 << 21)
#define D3DX_FILTER_SRGB_OUT         (2 << 21)
#define D3DX_FILTER_SRGB             (3 << 21)

typedef enum _D3DXIMAGE_FILEFORMAT
{
	D3DXIFF_BMP = 0,
	D3DXIFF_JPG = 1,
	D3DXIFF_TGA = 2,
	D3DXIFF_PNG = 3,
	D3DXIFF_DDS = 4,
	D3DXIFF_PPM = 5,
	D3DXIFF_DIB = 6,
	D3DXIFF_HDR = 7,
	D3DXIFF_PFM = 8,
	D3DXIFF_FORCE_DWORD = 0x7fffffff
} D3DXIMAGE_FILEFORMAT;


typedef HRESULT (WINAPI *D3DXLoadSurfaceFromMemoryProc) (LPDIRECT3DSURFACE9, CONST PALETTEENTRY *, CONST RECT *, LPCVOID, D3DFORMAT, UINT, CONST PALETTEENTRY *, CONST RECT *, DWORD, D3DCOLOR);
typedef HRESULT (WINAPI *D3DXSaveSurfaceToFileProc) (LPCSTR, D3DXIMAGE_FILEFORMAT, LPDIRECT3DSURFACE9, CONST PALETTEENTRY *, CONST RECT *);
typedef HRESULT (WINAPI *D3DXLoadSurfaceFromSurfaceProc) (LPDIRECT3DSURFACE9, CONST PALETTEENTRY *, CONST RECT *, LPDIRECT3DSURFACE9, CONST PALETTEENTRY *, CONST RECT *, DWORD, D3DCOLOR);

extern D3DXLoadSurfaceFromMemoryProc D3DXLoadSurfaceFromMemory;
extern D3DXSaveSurfaceToFileProc D3DXSaveSurfaceToFile;
extern D3DXLoadSurfaceFromSurfaceProc D3DXLoadSurfaceFromSurface;
#endif


/*

  skins will be outline flood filled and mip mapped
  pics and sprites with alpha will be outline flood filled
  pic won't be mip mapped

  model skin
  sprite frame
  wall texture
  pic

*/

typedef enum
{
	it_skin,
	it_sprite,
	it_wall,
	it_pic,
	it_sky
} imagetype_t;

typedef struct texrect_s
{
	float sl;
	float sh;
	float tl;
	float th;
} texrect_t;

typedef struct image_s
{
	// note - code in cl_ents.c assumes that this is the first struct member (by casting a struct image_s* to char* then doing a strcmp)
	// !!!!! so don't change it !!!!!
	char	name[MAX_QPATH];		// game path, including extension

	imagetype_t	type;
	int		width, height;			// source image
	int		registration_sequence;	// 0 = free
	struct	msurface_s *texturechain;
	IDirect3DTexture9 *texture;		// gl texture binding
	qboolean mipmap;
	int			framecount;			// == r_image_frame_count if this image is active in this frame
} image_t;


#define		MAX_GLTEXTURES	2048


//===================================================================

typedef enum
{
	rserr_ok,

	rserr_invalid_fullscreen,
	rserr_invalid_mode,

	rserr_unknown
} rserr_t;

#include "gl_model.h"

void R_BeginRendering (int *x, int *y, int *width, int *height);
void R_EndRendering (void);

void R_SetDefaultState (void);

typedef struct
{
	float	x, y, z;
	float	s, t;
	float	r, g, b;
} glvert_t;


#define	MAX_LBM_HEIGHT		480
#define BACKFACE_EPSILON	0.01
#define VDECL(Stream, Offset, Type, Usage, UsageIndex) {Stream, Offset, Type, D3DDECLMETHOD_DEFAULT, Usage, UsageIndex}


//====================================================

extern	image_t		gltextures[MAX_GLTEXTURES];
extern	int			numgltextures;

extern	image_t		*r_notexture;
extern	image_t		*r_blacktexture;
extern	image_t		*r_greytexture;
extern	image_t		*r_whitetexture;
extern	int			r_visframecount;
extern	int			r_framecount;
extern	cplane_t	frustum[4];
extern	int			c_brush_polys, c_alias_polys, c_draw_call;

// screen size info
extern	refdef_t	r_newrefdef;

extern mleaf_t *r_viewleaf[2];

extern	cvar_t	*r_norefresh;
extern	cvar_t	*r_lefthand;
extern	cvar_t	*r_drawentities;
extern	cvar_t	*r_drawworld;
extern	cvar_t	*r_speeds;
extern	cvar_t	*r_novis;
extern	cvar_t	*r_nocull;
extern	cvar_t	*r_lerpmodels;
extern	cvar_t	*r_lightlevel;	// FIXME: This is a HACK to get the client's light level
extern	cvar_t	*r_showbboxes;
extern	cvar_t	*r_shownullmodels;
extern	cvar_t	*r_fullbright;

extern	cvar_t	*gl_dynamic;
extern	cvar_t	*gl_clear;
extern	cvar_t	*gl_cull;
extern	cvar_t	*gl_poly;
extern	cvar_t	*gl_polyblend;
extern	cvar_t	*gl_modulate;
extern	cvar_t	*vid_gamma;
extern	cvar_t	*vid_brightness;
extern	cvar_t	*intensity;
extern	cvar_t	*gl_texturemode;
extern	cvar_t	*gl_textureanisotropy;
extern  cvar_t  *gl_lockpvs;
extern	cvar_t	*gl_lockfrustum;

void R_TranslatePlayerSkin (int playernum);
void R_LightPoint (vec3_t p, vec3_t color);
void R_DynamicLightPoint (vec3_t p, vec3_t color);
void R_SetLightLevel (void);


//====================================================================

extern	model_t	*r_worldmodel;

extern	unsigned	d_8to24table[256];

extern	int		r_registration_sequence;


void V_AddBlend (float r, float g, float b, float a, float *v_blend);

int 	R_Init (void);
void R_Shutdown (void);

void R_ScreenShot_f (void);
void R_DrawAliasModel (entity_t *e);
void R_DrawBrushModel (entity_t *e);
void R_DrawSpriteModel (entity_t *e);
void R_DrawBeam (entity_t *e);
void R_DrawNullModel (entity_t *e);
void R_DrawWorld (refdef_t *rd, entity_t *r_worldentity);
void R_DrawParticles (refdef_t *rd);
void R_InitTextures (void);
void Draw_InitLocal (void);
qboolean R_CullBox (vec3_t mins, vec3_t maxs);
qboolean R_CullEntityForBBox (entity_t *e, vec3_t *bbox, vec3_t mins, vec3_t maxs);
void R_MarkLeaves (refdef_t *rd);
void R_UpdateLightStyles (refdef_t *rd);

void R_DrawSolidChain (msurface_t *chain, image_t *image, const int pass);
void R_DrawWaterChain (msurface_t *chain);
void R_DrawSkyChain (msurface_t *chain);
image_t *R_TextureAnimation (mtexinfo_t *tex, int frame);

#define MAX_BATCHED_SURFINDEXES		0x100000
#define MAX_BATCHED_SURFACES		1024

typedef struct surfbatch_s
{
	msurface_t *surfaces[MAX_BATCHED_SURFACES];
	image_t *lastimage;
	int numindexes;
	int numsurfaces;
} surfbatch_t;

void R_NewImageFrame (void);
void R_BeginBatchingSurfaces (surfbatch_t *batch);
void R_BatchSurface (surfbatch_t *batch, msurface_t *surf, image_t *image);
void R_EndBatchingSurfaces (surfbatch_t *batch);

void R_SetupWaterTransform (refdef_t *rd);
void R_SetupSkyTransform (refdef_t *rd);
void R_DrawUnderwaterWarp (void);

void R_ChainSurface (entity_t *e, msurface_t *surf);
void R_DrawTextureChains (entity_t *e);
void R_ClearTextureChains (void);

void COM_StripExtension (char *in, char *out);

qboolean Draw_GetPicSize (int *w, int *h, char *name);
void Draw_Pic (int x, int y, char *name);
void Draw_StretchPic (int x, int y, int w, int h, char *name);
void Draw_ConsoleBackground (int x, int y, int w, int h, char *pic, float alpha);
void Draw_Char (int x, int y, int c);
void Draw_String (int x, int y, const char *str, int len, int xor);
void Draw_Field (int x, int y, int color, int width, int value);

void Draw_Fill (int x, int y, int w, int h, int c);
void Draw_FadeScreen (void);
void Draw_Clear (unsigned colour);
void Draw_StretchRaw (int cols, int rows, byte *data, int frame);

qboolean R_BeginFrame (void);
void R_EndFrame (void);

void R_SwapBuffers (int);
void R_SetCinematicPalette (const uint8_t *palette);

int	Draw_GetPalette (void);
void Draw_PolyBlend (float *color);
void Draw_RunPostProcessStep (funcptr_t PostProcessCallBack);

void R_FillTexture (IDirect3DTexture9 *tex, byte *data, int width, int height, unsigned *palette, qboolean alpha);
IDirect3DTexture9 *R_CreateTexture (int width, int height, qboolean mipmap, qboolean alpha);
void R_LoadSurfaceFromSurface (IDirect3DSurface9 *dst, IDirect3DSurface9 *src);

struct image_s *R_RegisterSkin (char *name);

// this just wraps ri->Scratch_AllocSize so that i can pass a custom allocator to LoadPCX.  Yuck.
void *R_Malloc (int size);

void R_LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *height, void *(*allocator) (int));
void R_LoadTGA (char *name, byte **pic, int *width, int *height, void *(*allocator) (int));
void R_LoadWAL (char *name, byte **pic, int *width, int *height, void *(*allocator) (int));
void R_FloodFillSkin (byte *skin, int skinwidth, int skinheight, unsigned *palette);
unsigned *R_Palette24To32 (const byte *pal24);

image_t *R_LoadPic (char *name, byte *pic, int width, int height, imagetype_t type, unsigned *palette);
image_t	*R_FindImage (char *name, imagetype_t type);
void R_TextureMode (char *string, int anisotropy);
void R_ImageList_f (void);

void R_InitImages (void);
void R_ShutdownImages (void);

void R_FreeUnusedImages (void);

void R_BeginBuildingSurfaces (void);
void R_RegisterSurface (msurface_t *surf);
void R_EndBuildingSurfaces (model_t *mod, dbsp_t *bsp);

typedef struct glconfig_s
{
	const char *renderer_string;
	const char *vendor_string;
	const char *version_string;
	const char *extensions_string;
} glconfig_t;


typedef struct glstate_s
{
	// cached states
	IDirect3DIndexBuffer9 *CurrentIB;
	IDirect3DVertexDeclaration9 *CurrentDecl;

	int StateBits;

	qboolean DeviceLost;
	qboolean fullscreen;
} glstate_t;


extern glconfig_t  gl_config;
extern glstate_t   gl_state;

/*
====================================================================

IMPORTED FUNCTIONS

====================================================================
*/

extern	refimport_t	*ri;


glmatrix *R_IdentityMatrix (glmatrix *m);
glmatrix *R_MultMatrix (glmatrix *out, glmatrix *m1, glmatrix *m2);
glmatrix *R_LoadMatrix (glmatrix *m, float _11, float _12, float _13, float _14, float _21, float _22, float _23, float _24, float _31, float _32, float _33, float _34, float _41, float _42, float _43, float _44);
glmatrix *R_FrustumMatrix (glmatrix *m, float fovx, float fovy);
glmatrix *R_OrthoMatrix (glmatrix *m, float left, float right, float bottom, float top, float zNear, float zFar);
glmatrix *R_TranslateMatrix (glmatrix *m, float x, float y, float z);
glmatrix *R_ScaleMatrix (glmatrix *m, float x, float y, float z);
glmatrix *R_RotateMatrix (glmatrix *m, float p, float y, float r);
glmatrix *R_RotateMatrixAxis (glmatrix *m, float angle, float x, float y, float z);
glmatrix *R_CameraMatrix (glmatrix *m, const float *origin, const float *angles);
void R_InverseTransform (glmatrix *m, float *out, const float *in);
void R_Transform (glmatrix *m, float *out, const float *in);

// keeping these so we can reuse them
extern glmatrix	r_view_matrix;
extern glmatrix	r_projection_matrix;
extern glmatrix	r_mvp_matrix;

// bboxes
void R_DrawBBoxBox (vec3_t *boxpoints);
void R_DrawBBoxMinMaxs (float *mins, float *maxs);

// alpha
void R_ClearAlphaObjects (void);
void R_AddAlphaObject (entity_t *e, msurface_t *surf, float alpha);
void R_DrawAlphaObjects (void);
void R_DrawSingleEntityOnList (entity_t *e);
void R_DrawAlphaSurfaces (msurface_t *surfs[], int numsurfs);

// state changes
void R_SetGL2D (void);
void R_SetTexture (UINT stage, image_t *image);
void R_SetCubeTexture (UINT stage, IDirect3DCubeTexture9 *texture);
void R_ReleaseTexture (image_t *image);
void R_SetVertexShader (IDirect3DVertexShader9 *vs);
void R_SetPixelShader (IDirect3DPixelShader9 *ps);
void R_SetVertexDeclaration (IDirect3DVertexDeclaration9 *decl);
void R_SetScissor (BOOL enable, RECT *sr);

void R_SetViewport (D3DVIEWPORT9 *vp);
void R_SetIndexBuffer (IDirect3DIndexBuffer9 *ib);
void R_SetVertexBuffer (UINT StreamNumber, IDirect3DVertexBuffer9 *pStreamData, UINT OffsetInBytes, UINT Stride);
void R_SetDPUPMode (void);


// lightmaps
void R_ClearLightmapChains (void);
void R_ReleaseLightmaps (int rs);
void R_DrawLightmappedChain (int pass);
void R_BuildLightmapChain (msurface_t *surf, int pass);
void R_SetLightmapTexCoords (float *lm, msurface_t *surf, float *vec);

void R_PushDlights (entity_t *e, struct mnode_s *headnode);
void R_RunDLights (refdef_t *rd);
BOOL R_BeginEntityLighting (entity_t *e, dlight_t *dl);
void R_EndEntityLighting (entity_t *e);
qboolean R_ShouldEntityHaveDlights (entity_t *e);

extern int r_dlightframecount;

void R_CreateMeshBuffers (model_t *mod, dmdl_t *hdr);
void R_CreateSpriteBuffers (model_t *mod);

void R_FreeUnusedMeshBuffers (int rs);
void R_FreeUnusedSpriteBuffers (int rs);

void R_RegisterMeshBuffers (model_t *mod);
void R_RegisterSpriteBuffers (model_t *mod);

void R_SetEntityState (entity_t *e);
void R_SetEntityTransform (glmatrix *localmatrix, int flags);
void R_SetEntityLighting (entity_t *e, float *shadelight);

#define RS_ALPHABLEND			(1 << 0)
#define RS_ADDITIVEBLEND		(1 << 1)
#define RS_DEPTHTEST			(1 << 2)
#define RS_DEPTHWRITE			(1 << 3)
#define RS_ALPHATEST			(1 << 4)
#define RS_CULLCCW				(1 << 5)
#define RS_CULLCW				(1 << 6)
#define RS_SUBTRACTBLEND		(1 << 7)
#define RS_WIREFRAME			(1 << 8)
#define RS_MAXIMUMBLEND			(1 << 9)

void R_SetRenderStates (int statebits);
void R_SetSamplerState (UINT stage, D3DTEXTUREFILTERTYPE minmagfilter, D3DTEXTUREFILTERTYPE mipfilter, D3DTEXTUREADDRESS addressmode, DWORD anisotropy);

