/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"
#include "menu_common.h"

/*
=============================================================================

LOADGAME MENU

=============================================================================
*/

#define	MAX_SAVEGAMES	15

static menuframework_s	s_savegame_menu;

static menuframework_s	s_loadgame_menu;
static menuaction_s		s_loadgame_actions[MAX_SAVEGAMES];

char		m_savestrings[MAX_SAVEGAMES][32];
qboolean m_savevalid[MAX_SAVEGAMES];

void M_HackSavestring (char *str)
{
	// hack the save string to 0-pad dates and convert to non-US format
	char *a = strstr (str, "/");
	char *b = strstr (str, ":");

	// if both '/' and ':' are present then we assume that it has a date and time field, and so we need to hack it
	if (a && b)
	{
		// hack the date
		char mm[2] = {a[-2], a[-1]};
		char dd[2] = {a[1], a[2]};

		if (mm[0] == ' ') mm[0] = '0';
		if (mm[1] == ' ') mm[1] = '0';
		if (dd[0] == ' ') dd[0] = '0';
		if (dd[1] == ' ') dd[1] = '0';

		a[-2] = dd[0];
		a[-1] = dd[1];
		a[1] = mm[0];
		a[2] = mm[1];

		// hack the time
		if (b[-2] == ' ') b[-2] = '0';
		if (b[-1] == ' ') b[-1] = '0';
		if (b[1] == ' ') b[1] = '0';
		if (b[2] == ' ') b[2] = '0';
	}
}


void Create_Savestrings (void)
{
	int		i;
	DECLARE_FILE (f);
	char	name[MAX_OSPATH];

	for (i = 0; i < MAX_SAVEGAMES; i++)
	{
		Com_sprintf (name, sizeof (name), "%s/save/save%i/server.ssv", FS_Gamedir (), i);

		if (!FS_FOpenRead (name, &f))
		{
			strcpy (m_savestrings[i], "<empty>");
			m_savevalid[i] = false;
		}
		else
		{
			FS_Read (m_savestrings[i], sizeof (m_savestrings[i]), &f);
			FS_FCloseFile (&f);
			M_HackSavestring (m_savestrings[i]);
			m_savevalid[i] = true;
		}
	}
}

void LoadGameCallback (void *self)
{
	menuaction_s *a = (menuaction_s *) self;

	if (m_savevalid[a->generic.localdata[0]])
		Cbuf_AddText (va ("load save%i\n", a->generic.localdata[0]));

	M_ForceMenuOff ();
}

void LoadGame_MenuInit (void)
{
	int i;

	s_loadgame_menu.x = viddef.width / 2 - 120;
	s_loadgame_menu.y = viddef.height / 2 - 58;
	s_loadgame_menu.nitems = 0;

	Create_Savestrings ();

	for (i = 0; i < MAX_SAVEGAMES; i++)
	{
		s_loadgame_actions[i].generic.name = m_savestrings[i];
		s_loadgame_actions[i].generic.flags = QMF_LEFT_JUSTIFY;
		s_loadgame_actions[i].generic.localdata[0] = i;
		s_loadgame_actions[i].generic.callback = LoadGameCallback;

		s_loadgame_actions[i].generic.x = 0;
		s_loadgame_actions[i].generic.y = (i) * 10;

		// separate from autosave
		if (i > 0) s_loadgame_actions[i].generic.y += 10;

		// grey-out empty slots
		if (!m_savevalid[i]) s_loadgame_actions[i].generic.flags |= QMF_GRAYED;

		s_loadgame_actions[i].generic.type = MTYPE_ACTION;

		Menu_AddItem (&s_loadgame_menu, &s_loadgame_actions[i]);
	}
}

void LoadGame_MenuDraw (void)
{
	M_Banner ("m_banner_load_game");
	//	Menu_AdjustCursor( &s_loadgame_menu, 1 );
	Menu_Draw (&s_loadgame_menu);
}

const char *LoadGame_MenuKey (int key)
{
	if (key == K_ESCAPE || key == K_ENTER)
	{
		s_savegame_menu.cursor = s_loadgame_menu.cursor - 1;

		if (s_savegame_menu.cursor < 0)
			s_savegame_menu.cursor = 0;
	}

	return Default_MenuKey (&s_loadgame_menu, key);
}

void M_Menu_LoadGame_f (void)
{
	LoadGame_MenuInit ();
	M_PushMenu (LoadGame_MenuDraw, LoadGame_MenuKey);
}


/*
=============================================================================

SAVEGAME MENU

=============================================================================
*/
static menuframework_s	s_savegame_menu;
static menuaction_s		s_savegame_actions[MAX_SAVEGAMES];

void SaveGameCallback (void *self)
{
	menuaction_s *a = (menuaction_s *) self;

	Cbuf_AddText (va ("save save%i\n", a->generic.localdata[0]));
	M_ForceMenuOff ();
}

void SaveGame_MenuDraw (void)
{
	M_Banner ("m_banner_save_game");
	Menu_AdjustCursor (&s_savegame_menu, 1);
	Menu_Draw (&s_savegame_menu);
}

void SaveGame_MenuInit (void)
{
	s_savegame_menu.x = viddef.width / 2 - 120;
	s_savegame_menu.y = viddef.height / 2 - 58;
	s_savegame_menu.nitems = 0;

	Create_Savestrings ();

	// don't include the autosave slot
	for (int i = 0; i < MAX_SAVEGAMES - 1; i++)
	{
		s_savegame_actions[i].generic.name = m_savestrings[i + 1];
		s_savegame_actions[i].generic.localdata[0] = i + 1;
		s_savegame_actions[i].generic.flags = QMF_LEFT_JUSTIFY;
		s_savegame_actions[i].generic.callback = SaveGameCallback;

		s_savegame_actions[i].generic.x = 0;
		s_savegame_actions[i].generic.y = (i) * 10;

		s_savegame_actions[i].generic.type = MTYPE_ACTION;

		// grey-out empty slots
		if (!m_savevalid[i + 1]) s_savegame_actions[i].generic.flags |= QMF_GRAYED;

		Menu_AddItem (&s_savegame_menu, &s_savegame_actions[i]);
	}
}

const char *SaveGame_MenuKey (int key)
{
	if (key == K_ENTER || key == K_ESCAPE)
	{
		s_loadgame_menu.cursor = s_savegame_menu.cursor - 1;

		if (s_loadgame_menu.cursor < 0)
			s_loadgame_menu.cursor = 0;
	}

	return Default_MenuKey (&s_savegame_menu, key);
}

void M_Menu_SaveGame_f (void)
{
	if (!Com_ServerState ())
		return;		// not playing a game

	SaveGame_MenuInit ();
	M_PushMenu (SaveGame_MenuDraw, SaveGame_MenuKey);
	Create_Savestrings ();
}


