/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// models.c -- model loading and caching

#include "gl_local.h"

// sprites are so small that it's not worth making cut-down m_ versions of the structs
extern model_t	*loadmodel;
extern int		modfilelen;

/*
==============================================================================

SPRITE MODELS

==============================================================================
*/

/*
=================
Mod_LoadSpriteModel
=================
*/
void Mod_LoadSpriteModel (model_t *mod, void *buffer)
{
	dsprite_t *sprin = (dsprite_t *) buffer;
	dsprite_t *sprout = (dsprite_t *) ri->Hunk_Alloc (modfilelen);

	sprout->ident = LittleLong (sprin->ident);
	sprout->version = LittleLong (sprin->version);
	sprout->numframes = LittleLong (sprin->numframes);

	if (sprout->version != SPRITE_VERSION) ri->Sys_Error (ERR_DROP, "%s has wrong version number (%i should be %i)", mod->name, sprout->version, SPRITE_VERSION);
	if (sprout->numframes > MAX_MD2SKINS) ri->Sys_Error (ERR_DROP, "%s has too many frames (%i > %i)", mod->name, sprout->numframes, MAX_MD2SKINS);

	// byte swap everything
	for (int i = 0; i < sprout->numframes; i++)
	{
		dsprframe_t *framein = &sprin->frames[i];
		dsprframe_t *frameout = &sprout->frames[i];

		frameout->width = LittleLong (framein->width);
		frameout->height = LittleLong (framein->height);
		frameout->origin_x = LittleLong (framein->origin_x);
		frameout->origin_y = LittleLong (framein->origin_y);

		memcpy (frameout->name, framein->name, MAX_SKINNAME);

		mod->skins[i] = R_FindImage (frameout->name, it_sprite);
	}

	mod->type = mod_sprite;

	R_CreateSpriteBuffers (mod);
}

